#ifndef __TIFF__
#define __TIFF__
#include "windows.h"

// TAG data type 
const short tiByte  = 0x01;
const short tiChar  = 0x02;
const short tiWord  = 0x03;
const short tiLong  = 0x04;
// TAG format 
const short ttSubFile			= 0x0FF;
const short ttImageWidth		= 0x100;
const short ttImageLength		= 0x101;
const short ttBitsPerSample	= 0x102;
const short ttCompression		= 0x103;	//1-none, 2-modif Haffman(only 2colors W-0,B-1),5 - LZW
											//32773 - for
const short ttPhotoMetric		= 0x106;
const short ttStripOffsets		= 0x111;
const short ttOrientation		= 0x112;
const short ttSamplesPerPixel	= 0x115;
const short	ttStripByteCounts = 0x117;//=1
const short ttMinSampleVal		= 0x118;
const short ttMaxSampleVal		= 0x119;
const short ttXResol				= 0x11a;
const short ttYResol				= 0x11b;
const short ttPlanarConfig		= 0x11C;
const short ttColorResUnit		= 0x12C;
const short ttColorResCurve	= 0x12D; //length=3*2^BitsPerSample
const short ttColorMap			= 0x140; //length=3*2^BitsPerSample
const short ttRowsPerStrip		= 0x116;


const short	bIntel = 0x4949;
const short	bMotor = 0x4D4D;

//possible errors
const int	ERROR_FILENF	= -1;
const int	ERROR_INVHEAD	= -2;
const int	ERROR_COMPRESS = -3;
const int	ERROR_PHOTINT	= -4;
const	int	ERROR_SMALLBUF	= -5;
const int	ERROR_INVFILE	= -6;
//struct of TIFF header	
struct	tagTiffHeader{
	short	ByteOrder;		// II = Intel / MM = Motorola
	short	VerID;			// always 0x2A
	long	FirstIFD;		// Offset of first IFD Blok
};

struct	tagIFDElement{
	short	Tag;
	short	Type;
			// 1 - byte, 2 - ASCIIZ, 3 - word,
							// 4 - dword/uword, 5 - 2*dword(numerator&denumerator)
	long	Length;		// length of data in chars, words, long depending of type
	long	DataOffs;	//	offset of data

};


struct	TIFFPict{
	tagTiffHeader	Header;
	short	BitsPerSample;						// number of bits in each pixel component
	short	Compression;						//
	char	ColorMap[1024][3];				//!palette, all R,all G, all B
	short	PhotomInterp;						//!=3 how to interpret imgae data
	long	Height;								//!
	long	Width;								//!
	short	PlanarConfig;						//!
	short	Predictor;							//only with Compress==1(LZW)
	short	ResUnits;							// 
	long	RowsPerStrip;						//!
	long	StripsPerImage;					//
	short	SamplesPerPixel;					//1 - g/s, b/w & palette, 3 - RGB
	long	StripByteCounts[1024];			//number of bytes in line
	long	StripOffsets;						//!offset of each strip in file
	int	PalLength;
	short	Palette[256*3];
	long	PalOffset;
	TIFFPict()
	{
		PalLength = 0;
		BitsPerSample =1;
		Compression = 1;
		PhotomInterp = 3;
		PlanarConfig = 1;
		// Predictor = 1; //only LZW
		ResUnits = 2;
		RowsPerStrip = 8192;
		StripsPerImage = (Width+RowsPerStrip-1)/RowsPerStrip;
		SamplesPerPixel = 3;
//		StripByteCounts = StripsPerImage;
//		StripOffsets = StripsPerImage;
	}
protected:
	int	CurFilePos;
	int	Lines;
	char	FileName[256];
	bool	PalOk;
public:
	// reads and stores information about
	//		given file in this structure
	int	OpenFile(char* FileName);
	// resets reading pointer
	void	ResetPict();
	// reads next line of picture to buf
	int	ReadNextLine(char* buf, int maxlen);
	// reads palette from file, specified in FileName, must be called after ReadHeader func
	int	ReadPalette();
	// draws picture on give dc, must be mach nig to fit whole picture
	int	DrawPict(HDC dc, int x0, int y0);
	//checkes current object
	bool	Check();
	//procedures for creating new TIFF files
protected:
	bool	IsCreate;
	bool	IsPal;
	long	BytesWritten;
public:
	int	CreateNewFile(char* FileName, int Width, int Height, char Coding=1, int BitPix=8);
	int	SetPalette(unsigned char* Palette, int PalSize=256);
	int	SetPalette(short* Palette, int PalSize=256);
	int	SaveLine(unsigned char* buf, int buflen=0);
	int	CloseFile();
};

#endif//__TIFF__
